package fta3;

import java.io.*;
import java.lang.Integer;
import java.util.*;

class FT2Exp {

    public static void main (String args[]) { 
        int w = args.length - 1;
        String p = args[0];
        ETree[] ts = new ETree[w];
        SimpleIO.promptln ("");
        SimpleIO.promptln ("Processing MobyFT files (project " 
                         + args[0] + "): ");
        SimpleIO.promptln ("");
        for (int i = 0; i < w; i++) {
            String f = args[i+1];
            SimpleIO.promptln (f + " ");
	}        
        for (int i = 0; i < w; i++) {
            String f = args[i+1];
            FTFile k = new FTFile (p, f); 
	    ts[i] = k.readFromMoby ();
	    k.writeOut (ts[i]);
	}
        ETree nt = ETree.linkETs (ts);
        ETree ft = ETree.fillET (nt);
        ETree gt = ETree.ordET (ft);
        CTree ct1 = Trans2.transET2CT (gt);
        CTree ct = CTree.ordCT (ct1);
	CTree.printCT (ct);
        RTree rt = Trans2.transCT2RT (ct);
        LgExp e = Trans2.transRT2Exp (rt);
        SimpleIO.promptln ("");
        SimpleIO.promptln ("");
        SimpleIO.promptln ("");
        SimpleIO.promptln ("Printing the logical representation of "
                     + "the generated tree: ");
        LgExp[] xs = Trans2.transCT2Exp (ct);
        LgExp.printExps (xs);
        SimpleIO.promptln ("");
        SimpleIO.promptln ("");
        SimpleIO.promptln ("");
        SimpleIO.promptln ("Printing the fault formula: ");
        SimpleIO.promptln ("");
        LgExp.printExp (e);
        SimpleIO.promptln ("");
        SimpleIO.promptln ("");
    }
}

class FTFile {

    String projectNm;
    String fileNm;
    BufferedReader inR;

    FTFile () {};
    FTFile (String p, String x) {
        projectNm = p;
	fileNm = x;
    };   

    public void writeOut (ETree nt) {
	try {
	  String fNam = projectNm + "." + fileNm + ".FTdata";
          ObjectOutputStream out = new ObjectOutputStream (
		    new FileOutputStream (fNam)); 
          out.writeObject (nt);
          out.close ();
          ObjectInputStream in = new ObjectInputStream (
		       new FileInputStream (fNam));
          ETree ft = (ETree) in.readObject ();
          in.close();

          SimpleIO.promptln ("");
          SimpleIO.promptln ("Printing the information extracted "
                               + "from file " + fileNm + ": ");
          SimpleIO.promptln ("");
          ETree.printET (ft);
       	} catch (Exception e) { 
            System.out.println("Error: " + e);
        }
    }

    public ETree readFromMoby () {
	String line;
        String fnm = extFileNm (fileNm);
        ETree nT = new ETree (projectNm, fnm);
        try {
	  inR = new BufferedReader (
			  new FileReader (fileNm));
          line = inR.readLine ();
	  while (line != null)
	      { nT = extractInfo (line, nT); 
                line = inR.readLine (); }
          inR.close();
        }
        catch (IOException e) {
          System.out.println(e);
        }   
        return nT;  
    };

    public static String extFileNm (String ln) {
        int n1 = ln.indexOf (".MIX.");
	int n2 = n1 + 5;
        String nid = ln.substring (n2);
        return nid;
    };

    public ETree extractInfo (String lm, ETree oT) { 
        String ln = rExtString (lm);
	if (ln.matches (".*eventtype:.*")) 
	    {return (ETree.addEventLn (oT, ln));} 
	else if (ln.matches (".*gatetype:.*"))
            {return (ETree.addGateLn (oT, ln));} 
        else if (ln.matches (".*MIXEdge:.*"))
	    {return (ETree.addEdgeLn (oT, ln));} 
        else if (ln.matches (".*FTTransferIn:.*"))
	    {return (ETree.addInLn (oT, ln));} 
        else if (ln.matches (".*FTTransferOut:.*"))
	    {return (ETree.addOutLn (oT, ln));} 
        else return oT;
    };

    public String rExtString (String l) { 
        String line = l;
        String fl = ""; 
        try {
          do {String xl = line.replace ('\n', ' ');
              String yl = xl.replace ('\r', ' ');
              fl = fl + " " + yl;
              if (line.matches (".* end:.*")) break;
              else continue;
          } while ((line = inR.readLine()) != null);
	} catch (Exception e) { 
             System.out.println("Error: " + e);
	}
        return fl;
    };
}

class Trans2 {

    public static LgExp transNode2Exp (TNode d) {
                String p = d.nName;
                String ft = d.nType;
                String ty = transType (ft);
                LgExp q = new LgExp (p);
                q.eDes = ty;
                return q;
		//                String q = "[" + p + "::" + ty + "]";
		//		return (new LgExp (q));
    
    }

    public static LgExp[] transCT2Exp (CTree t) {
        int n = t.cComps.length;
        LgExp[] xs = new LgExp[n];
        for (int i = 0; i < n; i++) {
            xs[i] = transComp2Exp (t.cComps[i]);
	}
        return xs;
    }

    public static LgExp transComp2Exp (TComp c) {
        int n = c.cCauses.length;
        LgSym s = transGate2Sym (c.cGate, n);
        LgExp k = transNode2Exp (c.cEffect);
        LgExp[] ys = new LgExp[n];
        for (int i = 0; i < n; i++) {
	    ys[i] = transNode2Exp (c.cCauses[i]);
	}
        LgExp x = LgExp.gtNC (s, ys, k);
        return x;
    }

    public static LgExp transRT2Exp (RTree t) {
        int n = t.tTail.length;
        LgSym s = transGate2Sym (t.tHead.cGate, n);
        LgExp[] ys = new LgExp[n];
        for (int i = 0; i < n; i++) {
            if (t.tTail[i].tTail.length == 0) {
		ys[i] = transNode2Exp (t.tHead.cCauses[i]);
	    }
            else ys[i] = transRT2Exp (t.tTail[i]);
	}
        LgExp x = new LgExp (s, ys);
        return x;
    }

    public static String transType (String x) {
        if (x.equals ("0")) return "X";
        else if (x.equals ("1")) return "U"; 
        else if (x.equals ("2")) return "B"; 
        else if (x.equals ("3")) return "C"; 
        else if (x.equals ("IN")) return "I"; 
        else if (x.equals ("OUT")) return "O"; 
        else return null;
    }

    public static LgSym transGate2Sym (TGate g, int n) {
        LgSym y = new LgSym ();
        String s = g.nType;
        if (s.equals ("0")) y = LgSym.and (n); // AND
        else if (s.equals ("1")) y = LgSym.or (n);  // OR
        else if (s.equals ("2")) y = LgSym.andC (n);  // ANDC
        else if (s.equals ("3")) y = LgSym.andAC (n);  // ANDAC
        else if (s.equals ("4")) y = LgSym.orC (n); // ORC
        else if (s.equals ("5")) y = LgSym.inh (n); // INH
        else if (s.equals ("6")) y = LgSym.inhC (n); // INHC
        else y = null;
        return y;
    }

    public static CTree transET2CT (ETree t) {
	CTree x = CTree.extCTree (t.tGates, t.tEdges);
        x.cProjectNm = t.tProjectNm;
        return x;
    }

    public static RTree transCT2RT (CTree t) {
        RTree[] rs = RTree.tcomp2rt (t.cComps);
        RTree nt = RTree.buildRT (rs);
        return nt;
    }
}



